/*      File:   file.c
        Author: Nancy Montanez
        Date:   September 24, 2000
        Description:    DMZ core API - File Manager
*/

#include <stdio.h>
#include <stdlib.h>
#include "dmzcore.h"

size_t dmz_readfile(char *filename, char **data)
{
	FILE *fp;
	size_t size = 0;

	*data = NULL;

	if ((fp = fopen(filename, "rb")) == NULL)
	{
		dmz_logerr("[core:file:dmz_readfile] cannot open file");
		return 0;
	}

	/* get the file size */
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	if((*data = (char *) dmz_malloc(size)) == NULL)
	{
		dmz_logerr("[core:file:dmz_readfile] cannot allocate memory");
		return 0;
	}

	if (fread(*data, size, 1, fp) != size)
	{
		dmz_logerr("[core:file:dmz_readfile] cannot read from file");
		return 0;
	}

	dmz_logstd("[core:file:dmz_readfile] file read in successfully."); 
	fclose(fp);

	return size;
}

size_t dmz_writefileStd(char *filename, void *fileptr, size_t size)
{
	FILE *fp;
	size_t n = 0;

	if ((fp = fopen(filename, "wb")) == NULL)
	{
		dmz_logerr("[core:file:dmz_writefile] cannot open file");
		return 0;
	}

	if ((n = fwrite(fileptr, size, 1, fp)) != size)
	{
		dmz_logerr("[core:file:dmz_writefile] cannot write to file");
		return ((n > 0) ? n : 0);
	}

	fclose(fp);
	dmz_logstd("[core:file:dmz_writefile] file written successfully");

	return n;
}
/*
DMZ_API char *dmz_readfileWin(char *filename)
{
	//Uh, can we do this?
	HANDLE filehandle;

	// here we will use Createfile ... whereever it is ...
	filehandle = CreateFile(filename, GENERIC_READ, 0, NULL, OPEN_EXISTING, 0, 0);
	//uh, reads from file, but then what?
	CloseHandle(filehandle);

}

DMZ_API char *dmz_writefileWin(char *filename)
{
	HANDLE filehandle;
	
	filehandle = CreateFile(filename, GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, 0);

	//okay, this is just weird
	CloseHandle(filehandle);
}
*/

