/*	File:	bitmap.cpp
	Author:	Alex Fedosov
	Date:	March 14, 2001
	Description:	code for loading bitmaps and creating surfaces
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "dmzgraph.h"
#include "dmzgraph_internal.h"

#define LOBYTE(w)	((unsigned char) (w)) 
#define HIBYTE(w)	((unsigned char) (((unsigned short) (w) >> 8) & 0xFF))

unsigned char* dmz_graph_load_bitmap(char* filename, int *width, int *height)
{
	BITMAPFILEHEADER tFileHeader;
	BITMAPINFOHEADER tInfoHeader;
	FILE *fp;
	unsigned char *data = NULL;
	int size = 0;

	fp = fopen(filename, "rb");
	if (fp == NULL)
	{
		return NULL;
	}
	
	fseek(fp, 0, SEEK_END);
	size = ftell(fp);
	fseek(fp, 0, SEEK_SET);

	size = size - sizeof (BITMAPFILEHEADER) - sizeof (BITMAPINFOHEADER);
	if (size <= 0)
	{
		fclose(fp);
		return NULL;
	}

	memset(&tFileHeader, 0, sizeof(BITMAPFILEHEADER));

	/* read in the bitmap file header */
	if (fread(&tFileHeader, sizeof(BITMAPFILEHEADER), 1, fp) <= 0)
	{
		fclose(fp);
		return NULL;
	}

	/* check file type */
	if (LOBYTE(tFileHeader.bfType) != 'B' ||
		HIBYTE(tFileHeader.bfType) != 'M')
	{
		fclose(fp);
		return NULL;
	}

	/* read in information header */
	memset(&tInfoHeader, 0, sizeof(BITMAPINFOHEADER));
	if (fread(&tInfoHeader, sizeof(BITMAPINFOHEADER), 1, fp) <= 0)
	{
		fclose (fp);
		return NULL;
	}

	/* check color depth */
	if (tInfoHeader.biBitCount != 24 && tInfoHeader.biBitCount != 8)
	{
		fclose(fp);
		return NULL;
	}

	data = (unsigned char *) malloc(size);

	if (data == NULL)
	{
		fclose(fp);
		return NULL;
	}

	if (fread(data, size, 1, fp) <= 0)
	{
		fclose(fp);
		free(data);
		return NULL;
	}

	if (width != NULL) *width = tInfoHeader.biWidth;
	if (height != NULL) *height = tInfoHeader.biHeight;

	return data;
}
