//--------------------------------------------------------------------------//
// Nancy Montanez
// Connection file (with accept, listen, and other helpers.)
// Created: Sometime in October
// Modified: Nov. 26, 2000
// Modified again: Dec 2, 2000
// Modified again: March 1, 2001 -- Multiplexing added
//--------------------------------------------------------------------------//
/* Main Headers		*/
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h> 
#include <sys/socket.h>

/* Network Headers 	*/
#include <netinet/in.h>
#include <arpa/inet.h>

//#include "dmzcore.h"
#include "dmz_net.h"
#include "dmz_net_internal.h"

/* Headers for locking mechanism */
//#include "sys/lock.h"

unsigned long argp = 1L;		/* nonzero enables nonblocking mode */
//--------------------------------------------------------------------------//
int dmz_avail_socket(void)
{
	
	int i;

	// Enter the critical section
	//mutex_lock(objlock);

	/* for(i = 0; i < MAX_CONNECT; i++)
	{
		if(sockets[i].s == INVALID_SOCKET)
		{
			sockets[i].s = ~INVALID_SOCKET;
			break;
		}
	}
	*/
	// From the windows version
	for (i = 1; i < MAX_CONNECT; i++)
	{
		if (sockets[i].s == INVALID_SOCKET)
			break;
	}

	//Leave the critical section
//	mutex_unlock(objlock);

	if (i == MAX_CONNECT)
		return DMZ_NET_TOO_MANY_SOCS;
	else 
		return i;
}

//--------------------------------------------------------------------------//
int dmz_connect(char *peer, int port)
{
	int nRet;
	int sock_index;
	socklen_t addrlen;		/* Length specified during connect*/
	
	addrlen = sizeof(struct sockaddr_in);

	if (!init_done)
		return DMZ_NET_NOT_INIT;

	// check for available sockets in the array
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;
	
	//Check for invalid peers sent in by the function call
	if ( (peer == NULL) || (port <= 0) )
		return DMZ_NET_INVALID_PARAM;
	
	//Initialize the socket parameters
	sockets[sock_index].sa.sin_family = PF_INET;
	sockets[sock_index].sa.sin_addr.s_addr = inet_addr(peer);
	sockets[sock_index].sa.sin_port = htons(port);

	// Create the socket
	sockets[sock_index].s = socket(PF_INET, SOCK_STREAM, 0);

	if (sockets[sock_index].s == INVALID_SOCKET) 
		return DMZ_NET_SOCKET_ERROR;

	// now, call WinSock's connect to initiate the connection 
	nRet = connect(sockets[sock_index].s, 
		(const struct sockaddr *) &sockets[sock_index].sa, 
				addrlen);
	
	if( nRet == SOCKET_ERROR)
		return DMZ_NET_NO_CONNECT;

	return sock_index;
}
//--------------------------------------------------------------------------//
//Function: dmz_listen
//

int dmz_listen(int port, int max)
{
	int nRet;
	int sock_index;
	int list_ret;
	
	socklen_t addrlen;		/* Length specified during connect*/
	
	addrlen = sizeof(struct sockaddr_in);

	if (!init_done)
		return DMZ_NET_NOT_INIT;

	// check for available sockets in the array
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;

	//Check the parameters that are passed in from the function
	if( (port <= 0) || (max <= 0) || (max > MAX_CONNECT) )
		return DMZ_NET_INVALID_PARAM;
	
	sockets[sock_index].s = socket(PF_INET, SOCK_STREAM, 0);

	if(sockets[sock_index].s == INVALID_SOCKET) 
		return DMZ_NET_SOCKET_ERROR;

	// Initialize the socket
	sockets[sock_index].sa.sin_family = AF_INET;
	sockets[sock_index].sa.sin_addr.s_addr = htonl(INADDR_ANY);
	sockets[sock_index].sa.sin_port = htons(port);

	// now, name the socket using bind 
	nRet = bind(sockets[sock_index].s, 
		(const struct sockaddr *) &sockets[sock_index].sa, 
			addrlen);

	if(nRet == SOCKET_ERROR)
		return DMZ_NET_BINDING_ERR;

	// max = backlog ..
	list_ret = listen(sockets[sock_index].s, max);
	
	if (list_ret == SOCKET_ERROR)
		return DMZ_NET_LISTEN_ERR;

	return sock_index;
}
//----------------------------------------
//Function: dmz_accept
//

int dmz_accept(int listen_index)
{
	int accepted;
	//int nRet;
	int sock_index;

	socklen_t addrlen;		/* Length specified during connect*/
	
	addrlen = sizeof(struct sockaddr_in);

	if (!init_done)
		return DMZ_NET_NOT_INIT;

	// check for available sockets in the array
	sock_index = dmz_avail_socket();

	if (sock_index < 0)
		return sock_index;
	
	// once we get some non-zero number, we can accept. 
	accepted = accept(sockets[listen_index].s, 
		(struct sockaddr *) &sockets[sock_index].sa, 
			&addrlen);	

	if (accepted == INVALID_SOCKET)
		return DMZ_NET_ACCEPT_ERR;

	sockets[sock_index].s = accepted;
	
	return sock_index;
}
//--------------------------------------------------------------------------//
//Function: dmz_disconnect
//

void dmz_disconnect(int id)
{
	close(sockets[id].s);

	sockets[id].s = INVALID_SOCKET;
}
//-----------------------------------------------------------------------
// Function: dmz_net_multiplex
int dmz_net_multiplex(int *read_array, int *write_array, int *accept_array,
	int *read_ids, int *write_ids, int *accept_ids, 
	struct timeval *timeout)
{
	fd_set read_set;
	fd_set write_set;
	fd_set error_set;
	
	int *read_arraycp;
	int *write_arraycp;
	int *accept_arraycp;
	
	//variables just for kicks
	int x, i, j, z, y;
	int rv, error = 0;
	//char buf[256];
	
	if ( ((read_array == NULL) && (read_ids != NULL))  || 
		((read_array != NULL) && (read_ids == NULL)))
	                error = 1;
	else if ( ((write_array == NULL) && (write_ids != NULL))  || 
		((write_array != NULL) && (write_ids == NULL)))
	                error = 1;
	else if ( ((accept_array == NULL) && (accept_ids != NULL))  || 
		((accept_array != NULL) && (accept_ids == NULL)))
	                error = 1;
	                                                                   
	// parameters are not correct, return back to calling function
	if (error) return DMZ_NET_INVALID_PARAM; 
	
	// copy the arrays into new arrays to preserve the original contents of the errors
	// first, set space for the new arrays   
	
	if (read_array != NULL)
		read_arraycp = (int *) malloc (sizeof(int) * (*read_ids));
	if (accept_array != NULL)
		accept_arraycp = (int *) malloc (sizeof(int) * (*accept_ids));
	if (write_array != NULL)
		write_arraycp = (int *) malloc (sizeof(int) * (*write_ids));
	
	FD_ZERO(&read_set);
	FD_ZERO(&write_set);
	FD_ZERO(&error_set);
	
	// set the read array
	if(read_array != NULL)
	{
		for(i = 0; i < *read_ids; i++)
			FD_SET (sockets[read_array[i]].s, &read_set);
	 }
	 
	 // set the accept array
	 if(accept_array != NULL)   
	 {
	 	for(i = 0; i < *accept_ids; i++)
	 		FD_SET(sockets[accept_array[i]].s, &read_set);
	 }
	 
	 // set the write array
	 if(write_array != NULL)   
	 {
	 	for(i = 0; i < *write_ids; i++)
	 		FD_SET(sockets[write_array[i]].s, &write_set);
	 }
	//before calling select, copy the arrays into the temp arrays
	
	if (read_ids)
		memcpy( (void *) read_arraycp, (void *) read_array, 
			(sizeof (int) *(*read_ids)) );
	if (accept_ids)
		memcpy( (void *) accept_arraycp, (void *) accept_array, 
			(sizeof (int) *(*accept_ids)) );		
	if (write_ids)
		memcpy( (void *) write_arraycp, (void *) write_array, 
			(sizeof (int) *(*write_ids)) );	
	
	// NOw that the arrays are set, call the function.
	rv = select(FD_SETSIZE, &read_set, &write_set, &error_set, NULL);
	
	//sprintf(buf, "return value of select: %d", rv);
	//dmz_logstd(buf);
	
	// Check the rerutnr value from select()
	
	if (rv == 0)
		return DMZ_NET_TRY_AGAIN;
	if (rv == -1)
		return DMZ_NET_SOCKET_ERROR;
	
	z = 0;
	
	if (read_ids)
	{
		for (j = 0; j < *read_ids; j++)
		{
			if(FD_ISSET(sockets[read_arraycp[j]].s, &read_set))
				read_array[z++] = read_arraycp[j];
		}
		*read_ids = z;
	}
	
	y = 0;
	
	if (accept_ids)
	{
		for (j = 0; j < *accept_ids; j++)
		{
			if(FD_ISSET(sockets[accept_arraycp[j]].s, &read_set))
				accept_array[y++] = accept_arraycp[j];
		}
		*accept_ids = y;
	}
	x = 0;
	
	if (write_ids)
	{
		for (j = 0; j < *write_ids; j++)
		{
			if(FD_ISSET(sockets[write_arraycp[j]].s, &write_set))
				write_array[x++] = write_arraycp[j];
		}
		*write_ids = x;
	}
	
	return x + y + z;	
}
//-----------------------------------------------------------------------------
// FUcntionL dmz_net_chmode
int dmz_net_chmode(int connid, DMZ_SOCKET_MODE mode)
{
	int nRet;
	
	if ( (connid < 0) || (connid > MAX_CONNECT))
		return DMZ_NET_INVALID_PARAM;
	if(mode == NONBLOCKING)
		argp = 1L;
	else if (mode == BLOCKING)
		argp = 0l;
	else 
		return DMZ_NET_INVALID_PARAM;

	nRet = ioctl(sockets[connid].s, FIONBIO, &argp);
	
	if (nRet == SOCKET_ERROR)
		return DMZ_NET_NONBLOCKING_ERR;
	return (sockets[connid].m = mode);
}	

char *dmz_net_getpeer(int connid)
{
	char *tempipaddr;

	if ((connid < 0) || (connid > MAX_CONNECT) || (sockets[connid].s == INVALID_SOCKET))
		return NULL;

	tempipaddr = inet_ntoa(sockets[connid].sa.sin_addr);
	
	return tempipaddr;
}
//---------------------END OF conn.cpp--------------------------------------//
