/*	File:	dmzgraph.h
	Author:	Alex Fedosov
	Date:	February 27, 2001
	Description:	alternate graphics library
*/

#ifndef __DMZ_GRAPH_H__
#define __DMZ_GRAPH_H__

#ifdef WIN32
	#ifdef __cplusplus
		#ifdef DMZGRAPH_EXPORTS
			#define DMZ_GRAPHICS_API extern "C" __declspec(dllexport)
		#else
			#define DMZ_GRAPHICS_API extern "C" __declspec(dllimport)
		#endif
	#else
		#ifdef DMZGRAPH_EXPORTS	
			#define DMZ_GRAPHICS_API __declspec(dllexport)
		#else
			#define DMZ_GRAPHICS_API __declspec(dllimport)
		#endif
	#endif
#else
	#define DMZ_GRAPHICS_API
#endif

DMZ_GRAPHICS_API int dmz_graph_init(void* hwnd);
DMZ_GRAPHICS_API int dmz_graph_kill(void);
DMZ_GRAPHICS_API int dmz_graph_set_mode(int width, int height, int depth);
DMZ_GRAPHICS_API int dmz_graph_flip(void);
DMZ_GRAPHICS_API int dmz_graph_load_image(char *filename,
										  int *width, int *height);
DMZ_GRAPHICS_API int dmz_graph_create_virtual(int width, int height);
DMZ_GRAPHICS_API int dmz_graph_blit(int imgID, int src_x, int src_y,
							int src_w, int src_h, int dst_x, int dst_y, int transparent);
DMZ_GRAPHICS_API int dmz_graph_virblit(int srcID, int dstID, int src_x, int src_y,
							int src_w, int src_h, int dst_x, int dst_y, int transparent);
DMZ_GRAPHICS_API unsigned char* dmz_graph_load_bitmap(char* filename,
												int *width, int *height);
DMZ_GRAPHICS_API int dmz_graph_screenshot(char *filename);
DMZ_GRAPHICS_API int dmz_graph_get_last_error(void);

typedef enum {
		DMZ_GRAPH_SUCCESS		=  1,	/* functions returned successfully */
		DMZ_GRAPH_LOAD_ERROR	= -1,	/* error opening specified file */
		DMZ_GRAPH_ALREADY_INIT	= -2,	/* graphics library already initialized */
		DMZ_GRAPH_DDRAW_ERROR	= -3,	/* DirectDraw Error */
		DMZ_GRAPH_NOT_INIT		= -4,	/* graphics library not yet initialized */
		DMZ_GRAPH_TOO_MANY		= -5,	/* too many images loaded */
		DMZ_GRAPH_MODE_ERROR	= -6,	/* could not switch to given mode */
		DMZ_GRAPH_INVALID_PARAM = -7,	/* invalid parameters passed in */
		DMZ_GRAPH_INIT_ERROR	= -8,	/* initialization error */
		DMZ_GRAPH_FILE_ERROR	= -9,	/* error writing file */
		DMZ_GRAPH_SDL_ERROR	= -10,	/* SDL error */
		DMZ_GRAPH_UNSUPPORTED	= -42	/* functionality not yet supported */
		
} DMZ_GRAPH_RV;

#endif
